#!/bin/bash

###################################################################
# Copyright (c) 2004 PPARC, CERN, on behalf of the EU EGEE project.
#
# Instantiates the configuration templates of this package.
#
# Author: Akos.Frohner@cern.ch
#
###################################################################

PACKAGENAME=glite-security-trustmanager
USAGE="$0 [--dryrun] [--silent] [install_root [config_values_filename [config_directory]]]"

SILENT_ECHO=echo

TEMP=$(getopt -o d --long dryrun --long silent -- "$@")
eval set -- "$TEMP"

while true; do
    case "$1" in
        -d|--dryrun)
            DRYRUN='echo Info:'
            shift
            ;;
        --silent)
            SILENT_ECHO='set IGNORE_STRING='
            shift
            ;;
        --)
            # end of options
            shift
            break
            ;;
    esac
done

#################################################################
# installation root directory
if [ "$1" == "" ]; then
    INSTALLROOT='/opt/glite'
    $SILENT_ECHO "Info: using default install root: $INSTALLROOT"
else
    INSTALLROOT=$1
    shift
fi
if [ ! -d "$INSTALLROOT" ]; then
    echo "Error: Please specify the installation root directory!"
    echo " (e.g. /opt/glite)"
    echo $USAGE
    exit 1
fi

#################################################################
# configuration file
if [ "$1" == "" ]; then
    VALUESFILE="$INSTALLROOT/etc/$PACKAGENAME/config.properties"
    $SILENT_ECHO "Info: using default configuration file: $VALUESFILE"
else
    VALUESFILE=$1
    shift
fi
if [ ! -f "$VALUESFILE" ]; then
    echo "Error: Please specify the configuration values file!"
    echo " (e.g. config.properties)"
    echo $USAGE
    exit 1
fi

#################################################################
# configuration directory
if [ "$1" == "" ]; then
    CONFIGDIR="$INSTALLROOT/etc/$PACKAGENAME"
    $SILENT_ECHO "Info: using default configuration directory: $CONFIGDIR"
else
    CONFIGDIR=$1
    shift
fi
if [ ! -d "$CONFIGDIR" ]; then
    echo "Error: Please specify the configuration directory!"
    echo " (e.g. /opt/glite/etc/$PACKAGENAME)"
    echo $USAGE
    exit 1
fi

#################################################################
# Tomcat directory
if [ -r "/etc/tomcat5/tomcat5.conf" ]; then
    source "/etc/tomcat5/tomcat5.conf"
    TCCONFIG_DIR="/etc/tomcat5"
else
    if [ -z "$CATALINA_HOME" ]; then
	echo "Please set the CATALINA_HOME environmental variable!"
	exit 2
    fi
    $SILENT_ECHO "Warning: /etc/tomcat5/tomcat5.conf does not exists -- non default install."
    TCCONFIG_DIR="${CATALINA_HOME}/conf"
fi

SERVERXML="${TCCONFIG_DIR}/server.xml"

if [ ! -f "$SERVERXML" ]; then
    echo "Tomcat server configuraton file does not exists: $SERVERXML"
    exit 2
fi


#################################################################
# parsing the properties file and building a sed script:
#
#  - strip the package name prefix and the dot
#  - escape slashes
#  - turn the equal sign, surrounded with whitespaces, into slash
#  - prefix the line with "s/"
#  - suffix the line with "/g;"
#
sedscript=$(grep "^$PACKAGENAME" $VALUESFILE | sed -e "s/^$PACKAGENAME\.//" | sed -e 's/\//\\\//g; s/[ \t]*=[ \t]*/@\//; s/^/s\/@/; s/$/\/g;/;')
#
# add custom patterns: @INSTALLROOT@ -> $INSTALLROOT
#
sedscript=${sedscript}" "$(echo "INSTALLROOT=$INSTALLROOT" | sed -e 's/\//\\\//g; s/[ \t]*=[ \t]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=${sedscript}" "$(echo "HOSTNAME="$(hostname -f) | sed -e 's/\//\\\//g; s/[ \t]*=[ \t]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=${sedscript}" "$(echo "CATALINA_HOME=$CATALINA_HOME" | sed -e 's/\//\\\//g; s/[ \t]*=[ \t]*/@\//; s/^/s\/@/; s/$/\/g;/;')

#################################################################
# template instantiation
TEMPLATEDIR="$INSTALLROOT/share/$PACKAGENAME"
if [ ! -d "$TEMPLATEDIR" ]; then
    echo "Error: template directory does not exist: $TEMPLATEDIR"
    echo $USAGE
    exit 3
fi

for template in $(ls $TEMPLATEDIR/*.template); do
    if [ ! -f "$template" ]; then
        echo "Error: $template file does not exist!"
        exit 3
    fi
    filename=$CONFIGDIR/$(basename $template .template)

    if [ -f "$filename" ]; then
        $SILENT_ECHO "Warning: $filename already exists! Saving old one as $filename.old."
        $DRYRUN mv $filename $filename.old
    fi
    
    # let be paranoid with passwords
    $DRYRUN touch $filename
    $DRYRUN chmod 0600 $filename
    # replace the autogenerated patterns
    $DRYRUN cat $template | sed -e "$sedscript" >$filename
done


#################################################################
# configuration
OLDSERVERXML=$SERVERXML.old-glite
ORIGSERVERXML=$SERVERXML.orig-glite
if [ ! -e $ORIGSERVERXML ]; then
    if [ -e $OLDSERVERXML ]; then
	$DRYRUN cp $OLDSERVERXML $ORIGSERVERXML
    else
	$DRYRUN cp $SERVERXML $ORIGSERVERXML
    fi
fi

#if [ -e $OLDSERVERXML ]; then
#    echo "Error: $OLDSERVERXML already exists, so the server is configured"
#    exit 4
#fi

$DRYRUN cp -p $SERVERXML $OLDSERVERXML
$DRYRUN cp $CONFIGDIR/server.xml $SERVERXML
if [ ! -z "$TOMCAT_USER" ]; then
    $DRYRUN chown $TOMCAT_USER $SERVERXML
fi

JARS="bcprov-jdk14-*.jar log4j-*.jar"
for jar in $JARS; do
    if [ ! -e "$CATALINA_HOME/server/lib/$jar" ]; then
        $DRYRUN cp $TEMPLATEDIR/$jar $CATALINA_HOME/server/lib/
    fi
done

JARS2="glite-security-trustmanager.jar glite-security-util-java.jar"
for jar in $JARS2; do
	if [ ! -e "$CATALINA_HOME/server/lib/$jar" ]; then
    	$DRYRUN cp $INSTALLROOT/share/java/$jar $CATALINA_HOME/server/lib/
	fi
done

log4j="log4j-trustmanager.properties"
if [ ! -e "${TCCONFIG_DIR}/$log4j" ]; then
    $DRYRUN cp $CONFIGDIR/$log4j ${TCCONFIG_DIR}/$log4j
fi

$SILENT_ECHO "Info: you can clean up using the following commands"
$SILENT_ECHO "      mv -f $OLDSERVERXML $SERVERXML"
for jar in $JARS $JARS2; do
    $SILENT_ECHO "      rm -f $CATALINA_HOME/server/lib/$jar"
done
$SILENT_ECHO "      rm -f $TCCONFIG_DIR/$log4j"
$SILENT_ECHO "      rm -f ${CONFIGDIR}/server.xml"

